function [U1,U2,W,C,R]=CRcaar(P,T,thrs1,inf1,copnum,itermethod)
% function [U1,U2,W,C,R]=CRcaar(P,T,thrs1,inf1,copnum,itermethod)
% optimal costs and strategies for the Cops + Adversarial Robber game
% it can handle 1 or 2 cops and use Jacobi or Gauss-Seidel value iteration
%
% P:		N-by-N transition prob. matrix of G (used only for node connectivity) 
% T:		max number of iterations
% thrsh1:	threshoild to break iteration
% inf1:		initial value for C
% copnum:	number of cops (allowed: 1 or 2)
% itermethod:	iteration method (1: Jacobi, 2: Gauss-Seidel)
%
% ONE COP CASE: 
% U1:		N-by-N optimal cop move matrix; 		U1(mc,mr) is optimal move starting from position (mc,mr)
% U2:		empty matrix
% W:		N-by-N optimal robber move matrix; 		W(mc,mr) is optimal move starting from position (mc,mr)
% C:		N-by-N optimal capture time matrix;		C(mc,mr) is OCT starting from position (mc,mr) and cop's turn
% R:		N-by-N optimal capture time matrix;		R(mc,mr) is OCT starting from position (mc,mr) and robber's turn
%
% TWO COPS CASE: 
% U1:		N-by-N-by-N optimal cop1 move matrix; 		U1(mc1,mc2,mr) is optimal move starting from position (mc1,mc2,mr)
% U2:		N-by-N-by-N optimal cop2 move matrix; 		U2(mc1,mc2,mr) is optimal move starting from position (mc1,mc2,mr)
% W:		N-by-N-by-N optimal robber move matrix;		W(mc1,mc2,mr)  is optimal move starting from position (mc1,mc2,mr)
% C:		N-by-N-by-N optimal capture time matrix;	C(mc1,mc2,mr)  is OCT starting from position (mc1,mc2,mr) and cops' turn
% R:		N-by-N-by-N optimal capture time matrix;	R(mc1,mc2,mr)  is OCT starting from position (mc1,mc2,mr) and robber's turn

U1=[]; U2=[]; W=[]; C=[]; R=[];

if 	copnum==1 & itermethod==1
	[U,W,C,R]=caar11(P,T,thrs1,inf1);
	U1=U; U2=[];
elseif 	copnum==1 & itermethod==2
	[U,W,C,R]=caar12(P,T,thrs1,inf1);
	U1=U; U2=[];
elseif 	copnum==2 & itermethod==1
	[U1,U2,W,C,R]=caar21(P,T,thrs1,inf1);
elseif 	copnum==2 & itermethod==2
	[U1,U2,W,C,R]=caar22(P,T,thrs1,inf1);
elseif	copnum~=1 & copnum~=2
	disp('Allowed number of cops: 1 or 2!')
elseif	itermethod~=1 & itermethod~=2
	disp('Allowed itermethod: 1 (Jacobi) or 2 (Gauss-Seidel)!')
else
	disp('Something is wrong!')
end
%==========================================================================================
function [U,W,C,R]=caar11(P,T,thrs1,inf1)
% function [U,W,C,R]=caar11(P,T,thrs1,inf1)
% One Cop Against Adversarial Robber. PARALLEL / JACOBI ITERATION
%
% P:		transition probability matrix for the RW
% T:		max number of iterations
% thrsh1:	threshoild to break iteration
% inf1:		initial value for C
% U:		optimal cop play starting from position (mc,mr)
% W:		optimal robber play starting from position (mc,mr)
% C:		expected capture time starting from position (mc,mr) with cop moving
% R:		expected capture time starting from position (mc,mr) with robber moving

N=size(P,1);
C=inf1*(ones(N,N)-eye(N));
R=inf1*(ones(N,N)-eye(N));
U=zeros(N,N);
W=zeros(N,N);
PP=P+eye(N);

for t=1:T
	Cold=C;
	Rold=R;
	
	Cnew=C;
	Unew=U;
	for mc=1:N
		for mr=1:N
			%if C(mc,mr)==inf1
			if	mr~=mc
				C1=1e9;
				U1=0;
				Nc=find(PP(mc,:)>0);
				for nc=Nc
					if 1+R(nc,mr)<C1;
						C1=1+R(nc,mr);
						U1=nc;
					end
				end
				Cnew(mc,mr)=C1;
				Unew(mc,mr)=U1;
			end
		end
	end
	C=Cnew;
	U=Unew;
	changeC=max(max(abs(C-Cold)));
	
	Rnew=R;
	Wnew=W;
	for mc=1:N
		for mr=1:N
			%if R(mc,mr)==inf1
			if	mr~=mc
				R1=-1e9;
				W1=0;
				Nr=find(PP(mr,:)>0);
				for nr=Nr
					if C(mc,nr)>R1;
						R1=C(mc,nr);
						W1=nr;
					end
				end
				Rnew(mc,mr)=R1;
				Wnew(mc,mr)=W1;
			end
		end
	end
	R=Rnew;
	W=Wnew;
	changeR=max(max(abs(R-Rold)));

	%disp(Cold)
	%disp(Rold)
	%pause

	change=max(changeC,changeR);
	fprintf(1,'%6i  %12.6f\n',t,change);
	if change<thrs1; break; end
end

%==========================================================================================
function [U,W,C,R]=caar12(P,T,thrs1,inf1)
% function [U,W,C,R]=caar12(P,T,thrs1,inf1)
% One Cop Against Adversarial Robber. PARALLEL / GAUSS-SEIDEL ITERATION
%
% P:		transition probability matrix for the RW
% T:		max number of iterations
% thrsh1:	threshoild to break iteration
% inf1:		initial value for C
% U:		optimal cop play starting from position (mc,mr)
% W:		optimal robber play starting from position (mc,mr)
% C:		expected capture time starting from position (mc,mr) with cop moving
% R:		expected capture time starting from position (mc,mr) with robber moving

N=size(P,1);
C=inf1*(ones(N,N)-eye(N));
R=inf1*(ones(N,N)-eye(N));
U=zeros(N,N);
W=zeros(N,N);
PP=P+eye(N);

for t=1:T
	Cold=C;
	Rold=R;
	
	for mc=1:N
		for mr=1:N
			%if C(mc,mr)==inf1
			if	mr~=mc
				C1=1e9;
				U1=0;
				Nc=find(PP(mc,:)>0);
				for nc=Nc
					if 1+R(nc,mr)<C1;
						C1=1+R(nc,mr);
						U1=nc;
					end
				end
				C(mc,mr)=C1;
				U(mc,mr)=U1;
			end
		end
	end
	changeC=max(max(abs(C-Cold)));
	
	for mc=1:N
		for mr=1:N
			%if R(mc,mr)==inf1
			if	mr~=mc
				R1=-1e9;
				W1=0;
				Nr=find(PP(mr,:)>0);
				for nr=Nr
					if C(mc,nr)>R1;
						R1=C(mc,nr);
						W1=nr;
					end
				end
				R(mc,mr)=R1;
				W(mc,mr)=W1;
			end
		end
	end
	changeR=max(max(abs(R-Rold)));

	%disp(Cold)
	%disp(Rold)
	%pause

	change=max(changeC,changeR);
	fprintf(1,'%6i  %12.6f\n',t,change);
	if change<thrs1; break; end
end

%==========================================================================================
function [U1,U2,W,C,R]=caar21(P,T,thrs1,inf1)
% function [U1,U2,W,C,R]=cadr21(P,T,thrs1,inf1)
% Two Cops Against Adversarial Robber. PARALLEL / JACOBI ITERATION
%
% P:		transition probability matrix for the RW
% T:		max number of iterations
% thrsh1:	threshold to break iteration
% inf1:		initial value for C
% U1:		optimal play for cop nr.1 starting from position (mc1,mc2,mr)
% U2:		optimal play for cop nr.2 starting from position (mc1,mc2,mr)
% C:		expected capture time starting from position (mc1,mc2,mr)

N=size(P,1);
C=inf1*ones(N,N,N);
R=inf1*ones(N,N,N);
for n1=1:N
	for n2=1:N
		C(n1,n2,n1)=0;
		C(n1,n2,n2)=0;
		R(n1,n2,n1)=0;
		R(n1,n2,n2)=0;
	end
end
U1=zeros(N,N,N);
U2=zeros(N,N,N);
W=zeros(N,N,N);
PP=P+eye(N);

for t=1:T
	Cold=C;
	Rold=R;

	Cnew=C;
	U1new=U1;
	U2new=U1;
	for mc1=1:N
		for mc2=1:N
			for mr=1:N
				%if C(mc1,mc2,mr)==inf1
				if mc1~=mr & mc2~=mr
					CC=1e15;
					UU1=0;
					Nc1=find(PP(mc1,:)>0);
					UU2=0;
					Nc2=find(PP(mc2,:)>0);
					for nc1=Nc1;
						for nc2=Nc2;
							if 1+R(nc1,nc2,mr)<CC
								CC=1+R(nc1,nc2,mr);
								UU1=nc1;
								UU2=nc2;
							end
						end
					end
					Cnew(mc1,mc2,mr)=CC;
					U1new(mc1,mc2,mr)=UU1;
					U2new(mc1,mc2,mr)=UU2;
				end
			end
		end
	end
	C=Cnew;
	U1=U1new;
	U2=U2new;
	changeC=max(max(max(abs(C-Cold))));

	Rnew=R;
	Wnew=W;
	for mc1=1:N
		for mc2=1:N
			for mr=1:N
				%if R(mc1,mc2,mr)==inf1
				if mc1~=mr & mc2~=mr
					RR1=-1e15;
					WW1=0;
					Nr=find(PP(mr,:)>0);
					for nr=Nr
						if C(mc1,mc2,nr)>RR1
							RR1=C(mc1,mc2,nr);
							WW1=nr;
						end
					end
					Rnew(mc1,mc2,mr)=RR1;
					Wnew(mc1,mc2,mr)=WW1;
				end
			end
		end
	end
	R=Rnew;
	W=Wnew;
	changeR=max(max(max(abs(R-Rold))));
	
	change=max(changeC,changeR);
	fprintf(1,'%6i  %12.6f\n',t,change);
	if change<thrs1; break; end
end

%==========================================================================================
function [U1,U2,W,C,R]=caar22(P,T,thrs1,inf1)
% function [U1,U2,W,C,R]=cadr21(P,T,thrs1,inf1)
% Two Cops Against Adversarial Robber. PARALLEL / JACOBI ITERATION
%
% P:		transition probability matrix for the RW
% T:		max number of iterations
% thrsh1:	threshold to break iteration
% inf1:		initial value for C
% U1:		optimal play for cop nr.1 starting from position (mc1,mc2,mr)
% U2:		optimal play for cop nr.2 starting from position (mc1,mc2,mr)
% C:		expected capture time starting from position (mc1,mc2,mr)

N=size(P,1);
C=inf1*ones(N,N,N);
R=inf1*ones(N,N,N);
for n1=1:N
	for n2=1:N
		C(n1,n2,n1)=0;
		C(n1,n2,n2)=0;
		R(n1,n2,n1)=0;
		R(n1,n2,n2)=0;
	end
end
U1=zeros(N,N,N);
U2=zeros(N,N,N);
W=zeros(N,N,N);
PP=P+eye(N);

for t=1:T
	Cold=C;
	Rold=R;

	for mc1=1:N
		for mc2=1:N
			for mr=1:N
				if mc1~=mr & mc2~=mr
					CC=1e15;
					UU1=0;
					Nc1=find(PP(mc1,:)>0);
					UU2=0;
					Nc2=find(PP(mc2,:)>0);
					for nc1=Nc1;
						for nc2=Nc2;
							if 1+R(nc1,nc2,mr)<CC
								CC=1+R(nc1,nc2,mr);
								UU1=nc1;
								UU2=nc2;
							end
						end
					end
					C(mc1,mc2,mr)=CC;
					U1(mc1,mc2,mr)=UU1;
					U2(mc1,mc2,mr)=UU2;
				end
			end
		end
	end
	changeC=max(max(max(abs(C-Cold))));

	for mc1=1:N
		for mc2=1:N
			for mr=1:N
				%if R(mc1,mc2,mr)==inf1
				if mc1~=mr & mc2~=mr
					RR1=-1e15;
					WW1=0;
					Nr=find(PP(mr,:)>0);
					for nr=Nr
						if C(mc1,mc2,nr)>RR1
							RR1=C(mc1,mc2,nr);
							WW1=nr;
						end
					end
					R(mc1,mc2,mr)=RR1;
					W(mc1,mc2,mr)=WW1;
				end
			end
		end
	end
	changeR=max(max(max(abs(R-Rold))));
	
	change=max(changeC,changeR);
	fprintf(1,'%6i  %12.6f\n',t,change);
	if change<thrs1; break; end
end

