function [U1,U2,C]=CRcadr(P,T,thrs1,inf1,copnum,itermethod)
% function [U1,U2,C]=CRcadr(P,T,thrs1,inf1,copnum,itermethod)
% optimal costs and strategies for the Cops + Drunk Robber game
% it can handle 1 or 2 cops and use Jacobi or Gauss-Seidel value iteration
%
% P:		N-by-N transition prob. matrix of G (used only for node connectivity) 
% T:		max number of iterations
% thrsh1:	threshoild to break iteration
% inf1:		initial value for C
% copnum:	number of cops (allowed: 1 or 2)
% itermethod:	iteration method (1: Jacobi, 2: Gauss-Seidel)
%
% ONE COP CASE: 
% U1:		N-by-N optimal cop move matrix; 		U1(mc,mr) is optimal move starting from position (mc,mr)
% U2:		empty matrix
% C:		N-by-N optimal capture time matrix;		C(mc,mr) is OCT starting from position (mc,mr) and cop's turn
%
% TWO COPS CASE: 
% U1:		N-by-N-by-N optimal cop1 move matrix; 		U1(mc1,mc2,mr) is optimal move starting from position (mc1,mc2,mr)
% U2:		N-by-N-by-N optimal cop2 move matrix; 		U2(mc1,mc2,mr) is optimal move starting from position (mc1,mc2,mr)
% C:		N-by-N-by-N optimal capture time matrix;	C(mc1,mc2,mr)  is OCT starting from position (mc1,mc2,mr) and cops' turn

if 	copnum==1 & itermethod==1
	[U,C]=cadr11(P,T,thrs1,inf1);
	U1=U; U2=[];
elseif 	copnum==1 & itermethod==2
	[U,C]=cadr12(P,T,thrs1,inf1);
	U1=U; U2=[];
elseif 	copnum==2 & itermethod==1
	[U1,U2,C]=cadr21(P,T,thrs1,inf1);
elseif 	copnum==2 & itermethod==2
	[U1,U2,C]=cadr22(P,T,thrs1,inf1);
elseif	copnum~=1 & copnum~=2
	disp('Allowed number of cops: 1 or 2!')
elseif	itermethod~=1 & itermethod~=2
	disp('Allowed itermethod: 1 (Jacobi) or 2 (Gauss-Seidel)!')
else
	disp('Something is wrong!')
end
%=========================================================================================
function [U,C]=cadr11(P,T,thrs1,inf1)
% function [U,C]=cadr11(P,T,thrs1,inf1)
% One Cop Against Drunk Robber. PARALLEL / JACOBI ITERATION
%
% P:		transition probability matrix for the RW
% T:		max number of iterations
% thrsh1:	threshoild to break iteration
% inf1:		initial value for C
% U:		optimal cop play starting from position (mc,mr)
% C:		expected capture time starting from position (mc,mr)

N=size(P,1);
C=inf1*(ones(N,N)-eye(N));
U=zeros(N,N);
PP=P;%+eye(N);

for t=1:T
	Cold=C;
	%disp(Cold); pause
	Cnew=C;
	Unew=U;
	for mc=1:N
		for mr=1:N
			if	mr~=mc
				C1=1e5;
				U1=0;
				Nc=find(PP(mc,:)>0);
				for nc=Nc
					C2=1;
					P2=P; P2(nc,:)=0; P2(:,nc)=0;
					Nr=find(P(mr,:)>0);
					for nr=Nr
						C2=C2+P2(mr,nr)*C(nc,nr);
					end
					if 	C2<C1
						C1=C2;
						U1=nc;
					end
				end
				Cnew(mc,mr)=C1;
				Unew(mc,mr)=U1;
			end
		end
	end
	C=Cnew;
	U=Unew;
	change=max(max(abs(C-Cold)));
	fprintf(1,'%6i  %12.6f\n',t,change);
	if change<thrs1; break; end
end

%=========================================================================================
function [U,C]=cadr12(P,T,thrs1,inf1)
% function [U,C]=cadr12(P,T,thrs1,inf1)
% One Cop Against Drunk Robber. PARALLEL / GAUSS-SEIDEL ITERATION
%
% P:		transition probability matrix for the RW
% T:		max number of iterations
% thrsh1:	threshoild to break iteration
% inf1:		initial value for C
% U:		optimal cop play starting from position (mc,mr)
% C:		expected capture time starting from position (mc,mr)

N=size(P,1);
C=inf1*(ones(N,N)-eye(N));
U=zeros(N,N);
PP=P;%+eye(N);

for t=1:T
	Cold=C;
	%disp(Cold); pause
	Cnew=C;
	Unew=U;
	for mc=1:N
		for mr=1:N
			if	mr~=mc
				C1=1e5;
				U1=0;
				Nc=find(PP(mc,:)>0);
				for nc=Nc
					C2=1;
					P1=P; P1(nc,:)=0; P1(:,nc)=0;
					Nr=find(P1(mr,:)>0);
					for nr=Nr
						C2=C2+P1(mr,nr)*C(nc,nr);
					end
					if 	C2<C1
						C1=C2;
						U1=nc;
					end
				end
				C(mc,mr)=C1;
				U(mc,mr)=U1;
			end
		end
	end
	change=max(max(abs(C-Cold)));
	fprintf(1,'%6i  %12.6f\n',t,change);
	if change<thrs1; break; end
end

%=========================================================================================
function [U1,U2,C]=cadr21(P,T,thrs1,inf1)
% function [U1,U2,C]=cadr21(P,T,thrs1,inf1)
% Two Cops Against Drunk Robber. PARALLEL / JACOBI ITERATION
%
% P:		transition probability matrix for the RW
% T:		max number of iterations
% thrsh1:	threshoild to break iteration
% inf1:		initial value for C
% U1:		optimal play for cop nr.1 starting from position (mc1,mc2,mr)
% U2:		optimal play for cop nr.2 starting from position (mc1,mc2,mr)
% C:		expected capture time starting from position (mc1,mc2,mr)

N=size(P,1);
C=inf1*ones(N,N,N);
for n1=1:N
	for n2=1:N
		C(n1,n2,n1)=0;
		C(n1,n2,n2)=0;
	end
end
U1=zeros(N,N,N);
U2=zeros(N,N,N);
PP=P;%+eye(N);

for t=1:T
	Cold=C;
	Cnew=C;
	U1new=U1;
	U2new=U2;
	for mc1=1:N
		for mc2=1:N
			for mr=1:N
				if mr~=mc1 & mr~=mc2
					CC1=1e5;
					UU1=0;
					UU2=0;

					Nc1=find(PP(mc1,:)>0);
					Nc2=find(PP(mc2,:)>0);
					for nc1=Nc1
						for nc2=Nc2
							CC2=1;
							P2=P; P2(nc1,:)=0; P2(:,nc1)=0; P2(nc2,:)=0; P2(:,nc2)=0;
							Nr=find(P(mr,:)>0);
							for nr=Nr
								CC2=CC2+P2(mr,nr)*C(nc1,nc2,nr);
							end
							if 	CC2<CC1
								CC1=CC2;
								UU1=nc1;
								UU2=nc2;
							end
						end
					end
					Cnew(mc1,mc2,mr)=CC1;
					U1new(mc1,mc2,mr)=UU1;
					U2new(mc1,mc2,mr)=UU2;
				end
			end
		end
	end
	C=Cnew;
	U1=U1new;
	U2=U2new;
	change=max(max(max(abs(C-Cold))));
	fprintf(1,'%6i  %12.6f\n',t,change);
	if change<thrs1; break; end
end

%=========================================================================================
function [U1,U2,C]=cadr22(P,T,thrs1,inf1)
% function [U1,U2,C]=cadr21(P,T,thrs1,inf1)
% Two Cops Against Drunk Robber. PARALLEL / JACOBI ITERATION
%
% P:		transition probability matrix for the RW
% T:		max number of iterations
% thrsh1:	threshoild to break iteration
% inf1:		initial value for C
% U1:		optimal play for cop nr.1 starting from position (mc1,mc2,mr)
% U2:		optimal play for cop nr.2 starting from position (mc1,mc2,mr)
% C:		expected capture time starting from position (mc1,mc2,mr)

N=size(P,1);
C=inf1*ones(N,N,N);
for n1=1:N
	for n2=1:N
		C(n1,n2,n1)=0;
		C(n1,n2,n2)=0;
	end
end
U1=zeros(N,N,N);
U2=zeros(N,N,N);
PP=P;%+eye(N);

for t=1:T
	Cold=C;
	for mc1=1:N
		for mc2=1:N
			for mr=1:N
				if mr~=mc1 & mr~=mc2
					CC1=1e5;
					UU1=0;
					UU2=0;

					Nc1=find(PP(mc1,:)>0);
					Nc2=find(PP(mc2,:)>0);
					for nc1=Nc1
						for nc2=Nc2
							CC2=1;
							P2=P; P2(nc1,:)=0; P2(:,nc1)=0; P2(nc2,:)=0; P2(:,nc2)=0;
							Nr=find(P(mr,:)>0);
							for nr=Nr
								CC2=CC2+P2(mr,nr)*C(nc1,nc2,nr);
							end
							if 	CC2<CC1
								CC1=CC2;
								UU1=nc1;
								UU2=nc2;
							end
						end
					end
					C(mc1,mc2,mr)=CC1;
					U1(mc1,mc2,mr)=UU1;
					U2(mc1,mc2,mr)=UU2;
				end
			end
		end
	end
	change=max(max(max(abs(C-Cold))));
	fprintf(1,'%6i  %12.6f\n',t,change);
	if change<thrs1; break; end
end

