% demo of the main functions of the CopsRobber package
% Ath. Kehagias and P. Pralt, 2011
%
clear all
disp('---------------------------------------------------------------------------------------------')
disp('                        Cops and Robber Games: a Matlab Implementation                       ')
disp('                               Ath. Kehagias and P. Pralat, 2011                             ')
disp('---------------------------------------------------------------------------------------------')
disp('This Matlab package deals with COPS AND ROBBER GAMES, which can be described as follows.')
disp(' ')
disp('On an undirected graph K cops try to capture a robber. The game is played in rounds; ')
disp('in each round first the cops and then the robber move from their current node to a neighboring one. ')
disp('The cops want to capture the robber, which happens as soon as the robber and at least one cop ')
disp('are in the same node; the cops play optimally to achieve this end. The robber can be ')
disp('either ADVERSARIAL (he plays optimally to avoid capture) or DRUNK (he performs a random walk on the graph).')
disp('For more details on the problem and methods to solve it, you can read our techincal report: ')
disp('Ath. Kehagias and P. Pralat, "Cops and Drunk Robbers: a Computational Approach".')
disp(' ')
disp('We will now go through the basic functions provided by the CopsRobbers package. ')
disp('(you can find detailed usage instructions in the file CopsRobbersHelp.pdf).')
disp(' ')
disp('---------------------------------------------------------------------------------------------')
disp('Let us compute expected and maximum capture time for a drunk robber and a fixed cop strategy.')
disp('We will use the command: [p,Texp,Tmax]=CRstrat(P,X);')
disp(' ')
disp('Hit any key to proceed  ...')
P=grf2P01('Edge01b.txt');					% for graph G, construct RW prob. trans. matrix from edge list
X=load('strat01b.txt');						% load a fixed strategy
[p,Texp,Tmax]=CRstrat(P,X);
disp(' ')
disp(['Expected capture time is ' num2str(Texp)])
disp(['Maximum  capture time is ' num2str(Tmax)])
disp(' ')
disp('Hit any key to proceed  ...'); pause
disp(' ')
disp('---------------------------------------------------------------------------------------------')
disp('Let us compute expected optimal cop and robber strategies for one cop and an adversarial robber.')
disp('We will use the command: [U1,U2,W,Ca,Ra]=CRcaar(P,T,epsilon,initial,copnum,itermethod);')
disp(' ')
disp('Hit any key to proceed  ...'); pause
disp(' ')
T=200;								% max number of iterations CAAR / CADR will perform
epsilon=0.01;							% termination criterion to break execution of CAAR / CADR
initial=0;							% initial value for C
copnum=1;							% use one cop
itermethod=1;							% use Jacobi iteration
[U1,U2,W,Ca,Ra]=CRcaar(P,T,epsilon,initial,copnum,itermethod); 	% run the CAAR algorithm
disp(' ')
disp('The optimal cop strategy is stored in U1, the optimal robber strategy is stored in W.')
disp('The optimal cop cost is stored in Ca, the optimal robber cost is stored in Ra.')
disp(' ')
disp('Hit any key to proceed  ...'); pause
disp(' ')
disp('Let us check if C and R satisfy the optimality equations.')
disp('We will use the command: [EC,ER]=CRcheq(Ca,Ra, P);')
disp(' ')
disp('Hit any key to proceed  ...'); pause
[EC,ER]=CRcheq(Ca,Ra, P);
disp(['Violations in C optimality equations: ' num2str(sum(sum(EC))) ' (from a total of ' num2str(length(P)^2) ' equations)'])
disp(['Violations in R optimality equations: ' num2str(sum(sum(ER))) ' (from a total of ' num2str(length(P)^2) ' equations)'])
disp(' ')
disp('Hit any key to proceed  ...'); pause
disp('---------------------------------------------------------------------------------------------')
disp('Let us compute expected optimal cop and robber strategies for TWO cops and an adversarial robber.')
disp('We will use the command: [U1,U2,W,C,R]=CRcaar(P,T,epsilon,initial,copnum,itermethod);')
disp(' ')
disp('Hit any key to proceed  ...'); pause
disp(' ')
copnum=2;
[U1,U2,W,C,R]=CRcaar(P,T,epsilon,initial,copnum,itermethod);
disp(' ')
disp('The optimal cop strategies is stored in U1 and U2; the optimal robber strategy is stored in W.')
disp('The optimal cop cost is stored in C, the optimal robber cost is stored in R.')
disp('Note that all of the above are N-by-N-by-N matrices (corr. to 2 cop and 1 robber coordinates).')
disp(' ')
disp('Hit any key to proceed  ...'); pause
disp(' ')
disp('Let us check if C and R satisfy the optimality equations.')
disp('We will use the command: [EC,ER]=CRcheq(C,R, P);')
disp(' ')
disp('Hit any key to proceed  ...'); pause
[EC,ER]=CRcheq(C,R, P);
disp([ sum(sum(sum(EC))) sum(sum(sum(ER)))])
%[EC,ER]=CRcheq(C,[], P);
disp(['Violations in C optimality equations: ' num2str(sum(sum(sum(EC)))) ' (from a total of ' num2str(length(P)^3) ' equations)'])
disp(['Violations in R optimality equations: ' num2str(sum(sum(sum(ER)))) ' (from a total of ' num2str(length(P)^3) ' equations)'])
disp(' ')
disp('Hit any key to proceed  ...'); pause
disp('---------------------------------------------------------------------------------------------')
disp('Let us compute expected optimal cop and robber strategies for ONE cop and a DRUNK robber.')
disp('We will use the command: [U1,U2,Cd]=CRcadr(P,T,epsilon,initial,copnum,itermethod);')
disp(' ')
disp('Hit any key to proceed  ...'); pause
disp(' ')
copnum=1;
[U1,U2,Cd]=CRcadr(P,T,epsilon,initial,copnum,itermethod);
disp(' ')
disp('Let us check if C satisfies the optimality equations.')
disp('We will use the command: [EC,ER]=CRcheq(Cd,[], P);')
disp(' ')
disp('Hit any key to proceed  ...'); pause
[EC,ER]=CRcheq(Cd,[], P);
disp(['Violations in C optimality equations: ' num2str(sum(sum(EC))) ' (from a total of ' num2str(length(P)^2) ' equations)'])
disp(' ')
disp('Hit any key to proceed  ...'); pause
disp('---------------------------------------------------------------------------------------------')
disp('Let us compute the cost of drunkenness for one cop and an adversarial vs. drunk robber.')
disp('We will use the command: [F,CT,DCT,mopta,moptd]=CRcod(Ca,Ra,Cd);')
disp(' ')
disp('Hit any key to proceed  ...'); pause
disp(' ')
[F,CT,DCT,mopta,moptd]=CRcod(Ca,Ra,Cd);
disp(' ')
disp(['Optimal capture time (for adversarial robber) is ' num2str(CT)])
disp(['Expected capture time (for drunk robber) is ' num2str(DCT)])
disp(['The cost of drunkenness is CT/DCT=' num2str(F)])
disp(' ')
disp('Hit any key to proceed  ...'); pause
disp('---------------------------------------------------------------------------------------------')
disp('Let us simulate a game of one cop against a drunk robber using the previously computed optimal cop strategy.')
disp('We will use the command: [X1,X2,Y,Z,CT]=CRsim(x10,x20,y0,P,U1,U2,[],Tmax,output);')
disp(' ')
x10=1;
x20=-1;
y0=15;
Tmax=100;
output=1;
disp('Hit any key to proceed  ...'); pause
disp('------------------------------');
disp('    Time  Cop Robber C-R Dist.');
disp('------------------------------');
[X1,X2,Y,Z,CT]=CRsim(x10,x20,y0,P,U1,U2,[],Tmax,output);
