function [X1,X2,Y,Z,CT]=CRsim(x10,x20,y0,P,U1,U2,W,Tmax,output)
% function [X1,X2,Y,Z,CT]=CRsim(x10,x20,y0,P,U1,U2,W,Tmax,output)
% simulate (one OR two) cops vs. (adversarial OR drunk) robber
%
% x10:		initial position of cop1
% x20:		initial position of cop2 			(use x20<0 for one cop play)
% y0:		initial position of robber
% P:		N-by-N transition prob. matrix of G 		(used only for node connectivity) 
% U1:		N-by-N optimal move matrix for cop1; 		U1(mc,mr) is optimal move starting from position (mc,mr)
% U2:		N-by-N optimal move matrix for cop2; 		(if one cop game, use anything) 
% W:		N-by-N optimal robber move matrix; 		(for drunk robber use W=[])
% Tmax:		max duration of game
% output:	1 (positions of cops+robber displayed) or 0 (positions of cops+robber not displayed)
%
% ONE COP CASE: 
% X1:		Tmax-by-1 cop position matrix
% X2: 		empty matrix
% Y:		Tmax-by-1 robber position matrix
% Z: 		Tmax-by-1 cop-to-robber distance matrix
% CT:		capture time
%
% TWO COPS CASE: 
% X1:		Tmax-by-1 cop1 position matrix
% X2: 		Tmax-by-1 cop2 position matrix
% Y:		Tmax-by-1 robber position matrix
% Z: 		Tmax-by-1 min cop-to-robber distance matrix
% CT:		capture time


X1=[]; X2=[]; Y=[]; Z=[]; CT=inf;

if	x20< 1 & length(W)>0					% one cop  + adversarial robber
	[X,Y,Z,CT]=sim03(x10,y0,P,U1,W,Tmax,output);
elseif	x20< 1 & length(W)==0					% one cop  + drunk robber
	[X,Y,Z,CT]=sim01(x10,y0,P,U1,Tmax,output);
elseif	x20>=1 & length(W)>0					% two cops + adversarial robber
	[X1,X2,Y,Z,CT]=sim04(x10,x20,y0,P,U1,U2,W,Tmax,output);
elseif	x20>=1 & length(W)==0					% two cops + drunk robber\
	[X1,X2,Y,Z,CT]=sim02(x10,x20,y0,P,U1,U2,Tmax,output);
else
	disp('Something is wrong!')
end
T=length(Z);
if	Z(T)>1
	CT=inf;
end
%======================================================================================
function [X,Y,Z,CT]=sim01(x0,y0,P,U,Tmax,output)
% function [X,Y,Z,CT]=sim01(x0,y0,P,U,Tmax,output)
% simulate ONE Cop vs. drunk Robber

N=length(P);
[D,S]=shpth01(P);

for n1=1:N
	PP(n1,1)=P(n1,1);
	for n2=2:N
		PP(n1,n2)=PP(n1,n2-1)+P(n1,n2);
	end
end
%disp(PP)

X(1,1)=x0;
Y(1,1)=y0;
Z(1,1)=D(x0,y0);
if X(1,1)==Y(1,1)
	Tmax=0;
	CT=0;
end
if output
	disp([0 X(1) Y(1) Z(1)])
end
for t=2:Tmax
	CT=t-1;
	i=X(t-1,1);
	j=Y(t-1,1);
	rnd1=rand(1,1);
	if	U(i,j)>0
		X(t,1)=U(i,j);
	else
		X(t,1)=X(t-1,1);
	end
	if X(t,1)==Y(t-1,1)
		Y(t,1)=X(t,1);
		Z(t,1)=D(X(t),Y(t));
		if output
			disp([t-1 X(t) Y(t) Z(t)])
		end
		break
	end
	Y(t,1)=1;
	for n=2:N
		if (PP(j,n-1)<rnd1) & (rnd1<=PP(j,n))
			Y(t,1)=n;
		end
	end
	Z(t,1)=D(X(t),Y(t));
	if output
		disp([t-1 X(t) Y(t) Z(t)])
	end
	if X(t,1)==Y(t,1)
		break
	end
end


%======================================================================================
function [X1,X2,Y,Z,CT]=sim02(x10,x20,y0,P,U1,U2,Tmax,output)
% function [X1,X2,Y,Z,CT]=sim02(x10,x20,y0,P,U1,U2,Tmax,output)
% simulate TWO Cops vs. drunk Robber

N=length(P);
[D,S]=shpth01(P);

for n1=1:N
	PP(n1,1)=P(n1,1);
	for n2=2:N
		PP(n1,n2)=PP(n1,n2-1)+P(n1,n2);
	end
end
%disp(PP)

X1(1,1)=x10;
X2(1,1)=x20;
Y(1,1)=y0;
Z(1,1)=min(D(x10,y0),D(x20,y0));
if X1(1,1)==Y(1,1) || X2(1,1)==Y(1,1)
	Tmax=0;
	CT=0;
end
if output
	disp([0 X1(1) X2(1) Y(1) Z(1)])
end
for t=2:Tmax
	CT=t-1;
	i1=X1(t-1,1);
	i2=X2(t-1,1);
	j=Y(t-1,1);
	rnd1=rand(1,1);
	if	U1(i1,i2,j)>0
		X1(t,1)=U1(i1,i2,j);
	else
		X1(t,1)=X1(t-1,1);
	end
	if	U2(i1,i2,j)>0
		X2(t,1)=U2(i1,i2,j);
	else
		X2(t,1)=X2(t-1,1);
	end
	if X1(t,1)==Y(t-1,1)
		Y(t,1)=X1(t,1);
		Z(t,1)=min(D(X1(t),Y(t)),D(X2(t),Y(t)));
		if output
			disp([t-1 X1(t) X2(t) Y(t) Z(t)])
		end
		break
	end
	if X2(t,1)==Y(t-1,1)
		Y(t,1)=X2(t,1);
		Z(t,1)=min(D(X1(t),Y(t)),D(X2(t),Y(t)));
		if output
			disp([t-1 X1(t) X2(t) Y(t) Z(t)])
		end
		break
	end
	Y(t,1)=1;
	for n=2:N
		if (PP(j,n-1)<rnd1) & (rnd1<=PP(j,n))
			Y(t,1)=n;
		end
	end
	Z(t,1)=min(D(X1(t),Y(t)),D(X2(t),Y(t)));
	if output
		disp([t-1 X1(t) X2(t) Y(t) Z(t)])
	end
	if X1(t,1)==Y(t,1)
		break
	end
	if X2(t,1)==Y(t,1)
		break
	end
end


%======================================================================================
function [X,Y,Z,CT]=sim03(x0,y0,P,U,W,Tmax,output)
% function [X,Y,Z,CT]=sim01(x0,y0,P,U,W,Tmax,output)
% simulate ONE Cop vs. adversarial Robber

N=length(P);
[D,S]=shpth01(P);

X(1,1)=x0;
Y(1,1)=y0;
Z(1,1)=D(x0,y0);
if X(1,1)==Y(1,1)
	Tmax=0;
	CT=0;
end
if output
	disp([0 X(1) Y(1) Z(1)])
end
for t=2:Tmax
	CT=t-1;
	i=X(t-1,1);
	j=Y(t-1,1);
	if	U(i,j)>0
		X(t,1)=U(i,j);
	else
		X(t,1)=X(t-1,1);
	end
	if X(t,1)==Y(t-1,1)
		Y(t,1)=X(t,1);
		Z(t,1)=D(X(t),Y(t));
		if output
			disp([t-1 X(t) Y(t) Z(t)])
		end
		break
	end
	i=X(t,1);
	if	W(i,j)>0
		Y(t,1)=W(i,j);
	else
		Y(t,1)=Y(t-1,1);
	end
	if X(t,1)==Y(t,1)
		Z(t,1)=D(X(t),Y(t));
		if output
			disp([t-1 X(t) Y(t) Z(t)])
		end
		break
	end
	Z(t,1)=D(X(t),Y(t));
	if output
		disp([t-1 X(t) Y(t) Z(t)])
	end
end


%======================================================================================
function [X1,X2,Y,Z,CT]=sim04(x10,x20,y0,P,U1,U2,W,Tmax,output)
% function [X1,X2,Y,Z,CT]=sim04(x10,x20,y0,P,U1,U2,W,Tmax,output)
% simulate TWO Cops vs. adversarial Robber

N=length(P);
[D,S]=shpth01(P);

X1(1,1)=x10;
X2(1,1)=x20;
Y(1,1)=y0;
Z(1,1)=min(D(x10,y0),D(x20,y0));
if X1(1,1)==Y(1,1)
	Tmax=0;
	CT=0;
end
if X2(1,1)==Y(1,1)
	Tmax=0;
	CT=0;
end
if output
	disp([0 X1(1) X2(1) Y(1) Z(1)])
end
for t=2:Tmax
	CT=t-1;
	i1=X1(t-1,1);
	i2=X2(t-1,1);
	j=Y(t-1,1);
	if	U1(i1,i2,j)>0
		X1(t,1)=U1(i1,i2,j);
	else
		X1(t,1)=X1(t-1,1);
	end
	i1=X1(t);
	if	U2(i1,i2,j)>0
		X2(t,1)=U2(i1,i2,j);
	else
		X2(t,1)=X2(t-1,1);
	end
	if X1(t,1)==Y(t-1,1)
		Y(t,1)=X1(t,1);
		Z(t,1)=D(X1(t),Y(t));
		if output
			disp([t-1 X1(t) X2(t) Y(t) Z(t)])
		end
		break
	end
	if X2(t,1)==Y(t-1,1)
		Y(t,1)=X2(t,1);
		Z(t,1)=D(X2(t),Y(t));
		if output
			disp([t-1 X1(t) X2(t) Y(t) Z(t)])
		end
		break
	end
	i2=X2(t);
	if	W(i1,i2,j)>0
		Y(t,1)=W(i1,i2,j);
	else
		Y(t,1)=Y(t-1,1);
	end
	if X1(t,1)==Y(t,1) || X2(t,1)==Y(t,1)
		Z(t,1)=min(D(X1(t),Y(t)),D(X2(t),Y(t)));
		if output
			disp([t-1 X1(t) X2(t) Y(t) Z(t)])
		end
		break
	end
	Z(t,1)=min(D(X1(t),Y(t)),D(X2(t),Y(t)));
	if output
		disp([t-1 X1(t) X2(t) Y(t) Z(t)])
	end
end

%====================================================================================================
function [D,S]=shpth01(P)
% function [D,S]=shpth01(P)
% compute node distance from trans.prob. matrix P

N=length(P);
A=P>0;
B=ones(N,N); 
for n1=1:N; 
	for n2=1:N; 
		if A(n1,n2)==0 
		B(n1,n2)=inf; 
		end
	end
	B(n1,n1)=0;
end
[D,S]=FloydSPR(B);


%% Floyd algorithm
function [S, P]=FloydSPR(AdjMax)
% *INPUT:* 
% AdjMax: Adjacent matrix that represents a weighted, directed graph
%
% *OUTPUT:*
% S: distance to destination node
% P: next hop node
%
% *DESCRIPTION*
% Given a input adjacent matrix (AdjMax) that represents a weighted, directed graph. 
% The function finds the shorest path from one vertex 'i' to another 'j'. 
% The return values includes a matrix (S) that denotes the shortest distance 
% between vertices 'i' and 'j', and a matrix (P) that denotes the next vertex 'k' 
% on the path from vertex 'i' to vertex 'j' 


N=min(length(AdjMax(:,1)),length(AdjMax(1,:)));
P=-1*ones(N,N);
S=AdjMax;

for k=1:N
    for i=1:N
        for j=1:N
            if S(i,k)==inf
                continue;
            end
            if S(k,j)==inf
                continue;
            end
            if S(i,j)>S(i,k)+S(k,j)
                if P(i,k)==-1
                    P(i,j)=k;   
                else
                    P(i,j)=P(i,k);
                end
                S(i,j)=S(i,k)+S(k,j);
            end
        end
    end
end

%% END
